@@ -327,6 +327,22 @@ func (e *WorkflowCallEvent) EventName() string {
 	return "workflow_call"
 }
 
+// ImageVersionEvent is image_version event configuration.
+// https://docs.github.com/en/actions/reference/workflows-and-actions/events-that-trigger-workflows#image_version_ready
+type ImageVersionEvent struct {
+	// Names is a list of names which match to the created image names.
+	Names []*String
+	// Versions is a list of names which match to the created image versions. Glob patterns are available.
+	Versions []*String
+	// Pos is a position in source.
+	Pos *Pos
+}
+
+// EventName returns name of the event to trigger this workflow.
+func (e *ImageVersionEvent) EventName() string {
+	return "image_version"
+}
+
 // PermissionScope is struct for respective permission scope like "issues", "checks", ...
 // https://docs.github.com/en/actions/security-for-github-actions/security-guides/automatic-token-authentication#permissions-for-the-github_token
 type PermissionScope struct {
@@ -733,6 +749,8 @@ type Credentials struct {
 	Username *String
 	// Password is password for authentication.
 	Password *String
+	// Expression is a string when expression syntax ${{ }} is used for this section.
+	Expression *String
 	// Pos is a position in source.
 	Pos *Pos
 }
@@ -838,6 +856,18 @@ type WorkflowCall struct {
 	InheritSecrets bool
 }
 
+// Snapshot is a struct to represent image snapshot at jobs.<job_id>.snapshot.
+// https://docs.github.com/en/actions/reference/workflows-and-actions/workflow-syntax#jobsjob_idsnapshot
+// https://docs.github.com/en/actions/how-tos/manage-runners/larger-runners/use-custom-images
+type Snapshot struct {
+	// ImageName is a name of the custom image.
+	ImageName *String
+	// Version is a version of the custom image.
+	Version *String
+	// If is a condition whether the custom image is used.
+	If *String
+}
+
 // Job is configuration of how to run a job.
 // https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#jobs
 type Job struct {
@@ -890,6 +920,9 @@ type Job struct {
 	// WorkflowCall is a workflow call by 'uses:'.
 	// https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#jobsjob_iduses
 	WorkflowCall *WorkflowCall
+	// Snapshot is a custom image snapshot.
+	// https://docs.github.com/en/actions/reference/workflows-and-actions/workflow-syntax#jobsjob_idsnapshot
+	Snapshot *Snapshot
 	// Pos is a position in source.
 	Pos *Pos
 }
