@@ -6,7 +6,9 @@ import (
 	"encoding/json"
 	"fmt"
 	"io"
+	"slices"
 	"strings"
+	"sync"
 	"text/template"
 
 	"github.com/fatih/color"
@@ -156,26 +158,17 @@ func (e *Error) getIndicator(line string) string {
 	return fmt.Sprintf("%s^%s", strings.Repeat(" ", sw), strings.Repeat("~", uw))
 }
 
-// ByErrorPosition is predicate for sort.Interface. It sorts errors slice by file path, line, and
-// column.
-type ByErrorPosition []*Error
-
-func (by ByErrorPosition) Len() int {
-	return len(by)
-}
-
-func (by ByErrorPosition) Less(i, j int) bool {
-	if c := strings.Compare(by[i].Filepath, by[j].Filepath); c != 0 {
-		return c < 0
+func compareErrors(lhs, rhs *Error) int {
+	if c := strings.Compare(lhs.Filepath, rhs.Filepath); c != 0 {
+		return c
 	}
-	if by[i].Line == by[j].Line {
-		return by[i].Column < by[j].Column
+	if lhs.Line != rhs.Line {
+		return lhs.Line - rhs.Line
 	}
-	return by[i].Line < by[j].Line
-}
-
-func (by ByErrorPosition) Swap(i, j int) {
-	by[i], by[j] = by[j], by[i]
+	if lhs.Column != rhs.Column {
+		return lhs.Column - rhs.Column
+	}
+	return strings.Compare(lhs.Message, rhs.Message)
 }
 
 // ErrorTemplateFields holds all fields to format one error message.
@@ -214,10 +207,37 @@ func unescapeBackslash(s string) string {
 	return r.Replace(s)
 }
 
+func toPascalCase(s string) string {
+	ss := strings.FieldsFunc(s, func(r rune) bool {
+		return !('a' <= r && r <= 'z' || 'A' <= r && r <= 'Z' || '0' <= r && r <= '9')
+	})
+	for i, s := range ss {
+		var c rune
+		for _, c = range s {
+			break
+		}
+		if 'a' <= c && c <= 'z' {
+			ss[i] = strings.ToUpper(s[:1]) + s[1:]
+		}
+	}
+	return strings.Join(ss, "")
+}
+
+type ruleTemplateFields struct {
+	Name        string
+	Description string
+}
+
+func compareRuleTemplateByName(lhs, rhs *ruleTemplateFields) int {
+	return strings.Compare(lhs.Name, rhs.Name)
+}
+
 // ErrorFormatter is a formatter to format a slice of ErrorTemplateFields. It is used for
 // formatting error messages with -format option.
 type ErrorFormatter struct {
-	temp *template.Template
+	temp    *template.Template
+	rules   map[string]*ruleTemplateFields
+	rulesMu sync.Mutex
 }
 
 // NewErrorFormatter creates new ErrorFormatter instance. Given format must contain at least one
@@ -226,6 +246,11 @@ func NewErrorFormatter(format string) (*ErrorFormatter, error) {
 	if !strings.Contains(format, "{{") {
 		return nil, fmt.Errorf("template to format error messages must contain at least one {{ }} placeholder: %s", format)
 	}
+
+	r := map[string]*ruleTemplateFields{
+		"syntax-check": {"syntax-check", "Checks for GitHub Actions workflow syntax"},
+	}
+
 	funcs := template.FuncMap(map[string]interface{}{
 		"json": func(data interface{}) (string, error) {
 			var b strings.Builder
@@ -238,12 +263,23 @@ func NewErrorFormatter(format string) (*ErrorFormatter, error) {
 		"replace": func(s string, oldnew ...string) string {
 			return strings.NewReplacer(oldnew...).Replace(s)
 		},
+		"toPascalCase": toPascalCase,
+		"getVersion":   getCommandVersion,
+		"allKinds": func() []*ruleTemplateFields {
+			ret := make([]*ruleTemplateFields, 0, len(r))
+			for _, e := range r {
+				ret = append(ret, e)
+			}
+			slices.SortFunc(ret, compareRuleTemplateByName)
+			return ret
+		},
 	})
 	t, err := template.New("error formatter").Funcs(funcs).Parse(unescapeBackslash(format))
 	if err != nil {
 		return nil, fmt.Errorf("template %q to format error messages could not be parsed: %w", format, err)
 	}
-	return &ErrorFormatter{t}, nil
+
+	return &ErrorFormatter{t, r, sync.Mutex{}}, nil
 }
 
 // Print formats the slice of template fields and prints it with given writer.
@@ -262,3 +298,17 @@ func (f *ErrorFormatter) PrintErrors(out io.Writer, errs []*Error, src []byte) e
 	}
 	return f.Print(out, t)
 }
+
+// RegisterRule registers the rule. Registered rules are used to get description and index of error
+// kinds when you use `kindDescription` or `kindIndex` functions in an error format template. This
+// method can be called multiple times safely in parallel.
+func (f *ErrorFormatter) RegisterRule(r Rule) {
+	// Synchronize access to f.rules (#370)
+	f.rulesMu.Lock()
+	defer f.rulesMu.Unlock()
+
+	n := r.Name()
+	if _, ok := f.rules[n]; !ok {
+		f.rules[n] = &ruleTemplateFields{n, r.Description()}
+	}
+}
