@@ -1,12 +1,14 @@
-// +build gofuzz
+//go:build gofuzz
+
 package actionlint_fuzz
 
 import (
 	"github.com/rhysd/actionlint"
-	"gopkg.in/yaml.v3"
+	"go.yaml.in/yaml/v4"
 )
 
 func canParseByGoYAML(data []byte) (ret bool) {
+	ret = true
 	defer func() {
 		if err := recover(); err != nil {
 			ret = false
@@ -14,7 +16,7 @@ func canParseByGoYAML(data []byte) (ret bool) {
 	}()
 	var n yaml.Node
 	yaml.Unmarshal(data, &n)
-	return true
+	return
 }
 
 func FuzzParse(data []byte) int {
