@@ -11,7 +11,7 @@ import (
 	"path/filepath"
 	"regexp"
 	"runtime"
-	"sort"
+	"slices"
 	"strings"
 	"testing"
 
@@ -106,7 +106,10 @@ func testFindAllWorkflowsInDir(subdir string) (string, []string, error) {
 
 func checkErrors(t *testing.T, outfile string, errs []*Error) {
 	t.Helper()
-	t.Log("Checking errors with", outfile)
+	t.Logf("Checking %d errors with %q", len(errs), outfile)
+	for i, err := range errs {
+		t.Logf("  Error[%d]: %s", i, err.Error())
+	}
 
 	expected := []string{}
 	{
@@ -123,7 +126,7 @@ func checkErrors(t *testing.T, outfile string, errs []*Error) {
 		}
 	}
 
-	sort.Stable(ByErrorPosition(errs))
+	slices.SortFunc(errs, compareErrors)
 
 	if len(errs) != len(expected) {
 		ms := make([]string, 0, len(errs))
@@ -404,6 +407,7 @@ func TestLinterFormatErrorMessageOK(t *testing.T) {
 			}
 
 			if diff := cmp.Diff(want, have); diff != "" {
+				t.Logf("have: %s", have)
 				t.Fatal(diff)
 			}
 		})
@@ -460,6 +464,7 @@ func TestLinterFormatErrorMessageInSARIF(t *testing.T) {
 	}
 
 	if diff := cmp.Diff(want, have); diff != "" {
+		t.Logf("have: %s", have)
 		t.Fatal(diff)
 	}
 }
