@@ -70,14 +70,17 @@ func (p *parser) errorf(n *yaml.Node, format string, args ...interface{}) {
 }
 
 func (p *parser) unexpectedKey(s *String, sec string, expected []string) {
+	if !strings.ContainsRune(sec, ' ') {
+		sec = fmt.Sprintf("%q section", sec)
+	}
 	l := len(expected)
 	var m string
 	if l == 1 {
-		m = fmt.Sprintf("expected %q key for %q section but got %q", expected[0], sec, s.Value)
+		m = fmt.Sprintf("expected %q key for %s but got %q", expected[0], sec, s.Value)
 	} else if l > 1 {
-		m = fmt.Sprintf("unexpected key %q for %q section. expected one of %v", s.Value, sec, sortedQuotes(expected))
+		m = fmt.Sprintf("unexpected key %q for %s. expected one of %v", s.Value, sec, sortedQuotes(expected))
 	} else {
-		m = fmt.Sprintf("unexpected key %q for %q section", s.Value, sec)
+		m = fmt.Sprintf("unexpected key %q for %s", s.Value, sec)
 	}
 	p.errorAt(s.Pos, m)
 }
@@ -558,37 +561,51 @@ func (p *parser) parseWorkflowCallEvent(pos *Pos, n *yaml.Node) *WorkflowCallEve
 	return ret
 }
 
-func (p *parser) parseEvents(pos *Pos, n *yaml.Node) []Event {
+func (p *parser) parseImageVersionEvent(pos *Pos, n *yaml.Node) *ImageVersionEvent {
+	ret := &ImageVersionEvent{Pos: pos}
+
+	for _, kv := range p.parseSectionMapping("image_version", n, true, true) {
+		switch kv.id {
+		case "names":
+			ret.Names = p.parseStringSequence("names", kv.val, false, false)
+		case "versions":
+			ret.Versions = p.parseStringSequence("versions", kv.val, false, false)
+		default:
+			p.unexpectedKey(kv.key, "image_version", []string{"names", "versions"})
+		}
+	}
+
+	return ret
+}
+
+func (p *parser) parseEventWithNoConfig(n *yaml.Node) Event {
+	s := p.parseString(n, false)
+	switch s.Value {
+	case "":
+		return nil
+	case "schedule":
+		p.error(n, "schedule event must be configured with mapping")
+		return nil
+	case "repository_dispatch":
+		return &RepositoryDispatchEvent{Pos: posAt(n)}
+	case "workflow_dispatch":
+		return &WorkflowDispatchEvent{Pos: posAt(n)}
+	case "workflow_call":
+		return &WorkflowCallEvent{Pos: posAt(n)}
+	case "image_version":
+		return &ImageVersionEvent{Pos: posAt(n)}
+	default:
+		return &WebhookEvent{Hook: s, Pos: posAt(n)}
+	}
+}
+
+func (p *parser) parseEvents(n *yaml.Node) []Event {
 	switch n.Kind {
 	case yaml.ScalarNode:
-		switch n.Value {
-		case "workflow_dispatch":
-			return []Event{
-				&WorkflowDispatchEvent{Pos: posAt(n)},
-			}
-		case "repository_dispatch":
-			return []Event{
-				&RepositoryDispatchEvent{Pos: posAt(n)},
-			}
-		case "schedule":
-			p.errorAt(pos, "schedule event must be configured with mapping")
-			return []Event{}
-		case "workflow_call":
-			return []Event{
-				&WorkflowCallEvent{Pos: posAt(n)},
-			}
-		default:
-			h := p.parseString(n, false)
-			if h.Value == "" {
-				return []Event{}
-			}
-			return []Event{
-				&WebhookEvent{
-					Hook: h,
-					Pos:  posAt(n),
-				},
-			}
+		if e := p.parseEventWithNoConfig(n); e != nil {
+			return []Event{e}
 		}
+		return []Event{}
 	case yaml.MappingNode:
 		kvs := p.parseSectionMapping("on", n, false, true)
 		ret := make([]Event, 0, len(kvs))
@@ -606,6 +623,8 @@ func (p *parser) parseEvents(pos *Pos, n *yaml.Node) []Event {
 				ret = append(ret, p.parseRepositoryDispatchEvent(pos, kv.val))
 			case "workflow_call":
 				ret = append(ret, p.parseWorkflowCallEvent(pos, kv.val))
+			case "image_version":
+				ret = append(ret, p.parseImageVersionEvent(pos, kv.val))
 			default:
 				ret = append(ret, p.parseWebhookEvent(kv.key, kv.val))
 			}
@@ -618,17 +637,8 @@ func (p *parser) parseEvents(pos *Pos, n *yaml.Node) []Event {
 		ret := make([]Event, 0, l)
 
 		for _, c := range n.Content {
-			if s := p.parseString(c, false); s != nil {
-				switch s.Value {
-				case "schedule", "repository_dispatch":
-					p.errorf(c, "%q event should not be listed in sequence. Use mapping for \"on\" section and configure the event as values of the mapping", s.Value)
-				case "workflow_dispatch":
-					ret = append(ret, &WorkflowDispatchEvent{Pos: posAt(c)})
-				case "workflow_call":
-					ret = append(ret, &WorkflowCallEvent{Pos: posAt(c)})
-				default:
-					ret = append(ret, &WebhookEvent{Hook: s, Pos: posAt(c)})
-				}
+			if e := p.parseEventWithNoConfig(c); e != nil {
+				ret = append(ret, e)
 			}
 		}
 
@@ -915,6 +925,33 @@ func (p *parser) parseStrategy(pos *Pos, n *yaml.Node) *Strategy {
 	return ret
 }
 
+func (p *parser) parseCredentials(pos *Pos, n *yaml.Node) *Credentials {
+	ret := &Credentials{Pos: pos}
+
+	if e := p.mayParseExpression(n); e != nil {
+		ret.Expression = e
+		return ret
+	}
+
+	for _, kv := range p.parseSectionMapping("credentials", n, false, true) {
+		switch kv.id {
+		case "username":
+			ret.Username = p.parseString(kv.val, false)
+		case "password":
+			ret.Password = p.parseString(kv.val, false)
+		default:
+			p.unexpectedKey(kv.key, "credentials", []string{"username", "password"})
+		}
+	}
+
+	if ret.Username == nil || ret.Password == nil {
+		p.errorAt(pos, "both \"username\" and \"password\" must be specified in \"credentials\" section")
+		return nil
+	}
+
+	return ret
+}
+
 // https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#jobsjob_idcontainer
 func (p *parser) parseContainer(sec string, pos *Pos, n *yaml.Node) *Container {
 	ret := &Container{Pos: pos}
@@ -928,22 +965,7 @@ func (p *parser) parseContainer(sec string, pos *Pos, n *yaml.Node) *Container {
 			case "image":
 				ret.Image = p.parseString(kv.val, false)
 			case "credentials":
-				cred := &Credentials{Pos: kv.key.Pos}
-				for _, c := range p.parseSectionMapping("credentials", kv.val, false, true) {
-					switch c.id {
-					case "username":
-						cred.Username = p.parseString(c.val, false)
-					case "password":
-						cred.Password = p.parseString(c.val, false)
-					default:
-						p.unexpectedKey(c.key, "credentials", []string{"username", "password"})
-					}
-				}
-				if cred.Username == nil || cred.Password == nil {
-					p.errorAt(kv.key.Pos, "both \"username\" and \"password\" must be specified in \"credentials\" section")
-					continue
-				}
-				ret.Credentials = cred
+				ret.Credentials = p.parseCredentials(kv.key.Pos, kv.val)
 			case "env":
 				ret.Env = p.parseEnv(kv.val)
 			case "ports":
@@ -965,6 +987,10 @@ func (p *parser) parseContainer(sec string, pos *Pos, n *yaml.Node) *Container {
 		}
 	}
 
+	if ret.Image == nil {
+		p.errorfAt(pos, "\"image\" is missing in %q section", sec)
+	}
+
 	return ret
 }
 
@@ -995,12 +1021,101 @@ func (p *parser) parseTimeoutMinutes(n *yaml.Node) *Float {
 	return f
 }
 
+func (p *parser) parseStepExecAction(keyvals []workflowKeyVal, isDocker bool, pos *Pos) *ExecAction {
+	ret := &ExecAction{}
+
+	for _, kv := range keyvals {
+		switch kv.id {
+		case "uses":
+			ret.Uses = p.parseString(kv.val, false)
+		case "with":
+			with := p.parseSectionMapping("with", kv.val, false, false)
+			ret.Inputs = make(map[string]*Input, len(with))
+			if isDocker {
+				for _, input := range with {
+					switch input.id {
+					case "entrypoint":
+						// https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#jobsjob_idstepswithentrypoint
+						ret.Entrypoint = p.parseString(input.val, false)
+					case "args":
+						// https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#jobsjob_idstepswithargs
+						ret.Args = p.parseString(input.val, true)
+					default:
+						ret.Inputs[input.id] = &Input{input.key, p.parseString(input.val, true)}
+					}
+				}
+			} else {
+				for _, input := range with {
+					ret.Inputs[input.id] = &Input{input.key, p.parseString(input.val, true)}
+				}
+			}
+		case "id", "if", "name", "env", "continue-on-error", "timeout-minutes":
+			// do nothing
+		default:
+			p.unexpectedKey(kv.key, "step to execute action", []string{
+				"id",
+				"if",
+				"name",
+				"env",
+				"continue-on-error",
+				"timeout-minutes",
+				"uses",
+				"with",
+			})
+		}
+	}
+
+	// Note: `ret.Uses` is never `nil` because `parseStep` checks `uses` key in advance
+	return ret
+}
+
+func (p *parser) parseStepExecRun(keyvals []workflowKeyVal, pos *Pos) *ExecRun {
+	ret := &ExecRun{}
+
+	for _, kv := range keyvals {
+		switch kv.id {
+		case "run":
+			ret.Run = p.parseString(kv.val, false)
+			ret.RunPos = kv.key.Pos
+		case "shell":
+			ret.Shell = p.parseString(kv.val, false)
+		case "working-directory":
+			ret.WorkingDirectory = p.parseString(kv.val, false)
+		case "id", "if", "name", "env", "continue-on-error", "timeout-minutes":
+			// do nothing
+		default:
+			p.unexpectedKey(kv.key, "step to run shell command", []string{
+				"id",
+				"if",
+				"name",
+				"env",
+				"continue-on-error",
+				"timeout-minutes",
+				"run",
+				"shell",
+				"working-directory",
+			})
+		}
+	}
+
+	// Note: `ret.Run` is never `nil` because `parseStep` checks `run` key in advance
+	return ret
+}
+
 // https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#jobsjob_idsteps
 func (p *parser) parseStep(n *yaml.Node) *Step {
 	ret := &Step{Pos: posAt(n)}
-	var workDir *String
 
-	for _, kv := range p.parseMapping("element of \"steps\" section", n, false, true) {
+	const (
+		isUnknown = iota
+		isAction
+		isDocker
+		isRun
+	)
+
+	kind := isUnknown
+	kvs := p.parseMapping("element of \"steps\" section", n, false, true)
+	for _, kv := range kvs {
 		switch kv.id {
 		case "id":
 			ret.ID = p.parseString(kv.val, false)
@@ -1014,89 +1129,23 @@ func (p *parser) parseStep(n *yaml.Node) *Step {
 			ret.ContinueOnError = p.parseBool(kv.val)
 		case "timeout-minutes":
 			ret.TimeoutMinutes = p.parseTimeoutMinutes(kv.val)
-		case "uses", "with":
-			var exec *ExecAction
-			if ret.Exec == nil {
-				exec = &ExecAction{}
-			} else if e, ok := ret.Exec.(*ExecAction); ok {
-				exec = e
+		case "uses":
+			if strings.HasPrefix(kv.val.Value, "docker://") {
+				kind = isDocker
 			} else {
-				p.errorfAt(kv.key.Pos, "this step is for running shell command since it contains at least one of \"run\", \"shell\" keys, but also contains %q key which is used for running action", kv.key.Value)
-				continue
+				kind = isAction
 			}
-			if kv.id == "uses" {
-				exec.Uses = p.parseString(kv.val, false)
-			} else {
-				// kv.key == "with"
-				with := p.parseSectionMapping("with", kv.val, false, false)
-				exec.Inputs = make(map[string]*Input, len(with))
-				for _, input := range with {
-					switch input.id {
-					case "entrypoint":
-						// https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#jobsjob_idstepswithentrypoint
-						exec.Entrypoint = p.parseString(input.val, false)
-					case "args":
-						// https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#jobsjob_idstepswithargs
-						exec.Args = p.parseString(input.val, true)
-					default:
-						exec.Inputs[input.id] = &Input{input.key, p.parseString(input.val, true)}
-					}
-				}
-			}
-			ret.Exec = exec
-		case "run", "shell":
-			var exec *ExecRun
-			if ret.Exec == nil {
-				exec = &ExecRun{}
-			} else if e, ok := ret.Exec.(*ExecRun); ok {
-				exec = e
-			} else {
-				p.errorfAt(kv.key.Pos, "this step is for running action since it contains at least one of \"uses\", \"with\" keys, but also contains %q key which is used for running shell command", kv.key.Value)
-				continue
-			}
-			switch kv.id {
-			case "run":
-				exec.Run = p.parseString(kv.val, false)
-				exec.RunPos = kv.key.Pos
-			case "shell":
-				exec.Shell = p.parseString(kv.val, false)
-			}
-			exec.WorkingDirectory = workDir
-			ret.Exec = exec
-		case "working-directory":
-			workDir = p.parseString(kv.val, false)
-			if e, ok := ret.Exec.(*ExecRun); ok {
-				e.WorkingDirectory = workDir
-			}
-		default:
-			p.unexpectedKey(kv.key, "step", []string{
-				"id",
-				"if",
-				"name",
-				"env",
-				"continue-on-error",
-				"timeout-minutes",
-				"uses",
-				"with",
-				"run",
-				"working-directory",
-				"shell",
-			})
+		case "run":
+			kind = isRun
+			// Note: Unexpected keys are checked in parseStepExecAction or parseStepExecRun later
 		}
 	}
 
-	switch e := ret.Exec.(type) {
-	case *ExecAction:
-		if e.Uses == nil {
-			p.error(n, "\"uses\" is required to run action in step")
-		}
-		if workDir != nil {
-			p.errorAt(workDir.Pos, "\"working-directory\" is not available with \"uses\". it is only available with \"run\"")
-		}
-	case *ExecRun:
-		if e.Run == nil {
-			p.error(n, "\"run\" is required to run script in step")
-		}
+	switch kind {
+	case isAction, isDocker:
+		ret.Exec = p.parseStepExecAction(kvs, kind == isDocker, posAt(n))
+	case isRun:
+		ret.Exec = p.parseStepExecRun(kvs, posAt(n))
 	default:
 		p.error(n, "step must run script with \"run\" section or run action with \"uses\" section")
 	}
@@ -1151,6 +1200,34 @@ func (p *parser) parseRunsOn(n *yaml.Node) *Runner {
 	return r
 }
 
+func (p *parser) parseSnapshot(pos *Pos, n *yaml.Node) *Snapshot {
+	switch n.Kind {
+	case yaml.ScalarNode:
+		return &Snapshot{ImageName: p.parseString(n, false)}
+	case yaml.MappingNode:
+		ret := &Snapshot{}
+		for _, kv := range p.parseSectionMapping("on", n, false, true) {
+			switch kv.id {
+			case "image-name":
+				ret.ImageName = p.parseString(kv.val, false)
+			case "version":
+				ret.Version = p.parseString(kv.val, false)
+			case "if":
+				ret.If = p.parseString(kv.val, false)
+			default:
+				p.unexpectedKey(kv.key, "snapshot", []string{"image-name", "version", "if"})
+			}
+		}
+		if ret.ImageName == nil {
+			p.errorAt(pos, "\"snapshot\" section must have \"image-name\" configuration")
+		}
+		return ret
+	default:
+		p.errorf(n, "\"snapshot\" section value must be string or mapping but found %s node", nodeKindName(n.Kind))
+		return nil
+	}
+}
+
 // https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#jobsjob_id
 func (p *parser) parseJob(id *String, n *yaml.Node) *Job {
 	ret := &Job{ID: id, Pos: id.Pos}
@@ -1255,6 +1332,8 @@ func (p *parser) parseJob(id *String, n *yaml.Node) *Job {
 				}
 			}
 			callOnlyKey = k
+		case "snapshot":
+			ret.Snapshot = p.parseSnapshot(k.Pos, v)
 		default:
 			p.unexpectedKey(kv.key, "job", []string{
 				"name",
@@ -1276,6 +1355,7 @@ func (p *parser) parseJob(id *String, n *yaml.Node) *Job {
 				"uses",
 				"with",
 				"secrets",
+				"snapshot",
 			})
 		}
 	}
@@ -1344,7 +1424,7 @@ func (p *parser) parse(n *yaml.Node) *Workflow {
 		case "name":
 			w.Name = p.parseString(v, true)
 		case "on":
-			w.On = p.parseEvents(k.Pos, v)
+			w.On = p.parseEvents(v)
 		case "permissions":
 			w.Permissions = p.parsePermissions(k.Pos, v)
 		case "env":
@@ -1404,8 +1484,10 @@ func handleYAMLUnmarshalError(err error) []*Error {
 
 	var m string
 	var l int
+	var c int
 	if pe, ok := err.(*yaml.ParserError); ok {
 		l = pe.Line
+		c = pe.Column
 		m = pe.Message
 	} else {
 		m = err.Error() // Fallback. I believe this line should be unreachable
@@ -1414,6 +1496,7 @@ func handleYAMLUnmarshalError(err error) []*Error {
 		Message: fmt.Sprintf("could not parse as YAML: %s", m),
 		Kind:    "syntax-check",
 		Line:    l,
+		Column:  c,
 	}}
 }
 
