@@ -1,6 +1,8 @@
 package actionlint
 
 import (
+	"maps"
+	"slices"
 	"strconv"
 	"strings"
 )
@@ -159,6 +161,9 @@ func (rule *RuleExpression) VisitWorkflowPre(n *Workflow) error {
 				rule.checkString(o.Description, "")
 				// o.Value will be checked in VisitWorkflowPost
 			}
+		case *ImageVersionEvent:
+			rule.checkStrings(e.Names, "")
+			rule.checkStrings(e.Versions, "")
 		}
 	}
 
@@ -246,6 +251,7 @@ func (rule *RuleExpression) VisitJobPre(n *Job) error {
 	}
 
 	rule.checkWorkflowCall(n.WorkflowCall)
+	rule.checkSnapshot(n.Snapshot)
 
 	rule.stepsTy = NewEmptyStrictObjectType()
 
@@ -290,8 +296,8 @@ func (rule *RuleExpression) VisitStep(n *Step) error {
 				rule.checkString(i.Value, "jobs.<job_id>.steps.with")
 			}
 		}
-		rule.checkString(e.Entrypoint, "")
-		rule.checkString(e.Args, "")
+		rule.checkString(e.Entrypoint, "jobs.<job_id>.steps.with")
+		rule.checkString(e.Args, "jobs.<job_id>.steps.with")
 		spec = e.Uses
 	}
 
@@ -482,10 +488,14 @@ func (rule *RuleExpression) checkContainer(c *Container, workflowKey, childWorkf
 	rule.checkString(c.Image, workflowKey)
 	if c.Credentials != nil {
 		k := childWorkflowKey + ".credentials" // e.g. jobs.<job_id>.container.credentials
-		rule.checkString(c.Credentials.Username, k)
-		rule.checkString(c.Credentials.Password, k)
+		if c.Credentials.Expression != nil {
+			rule.checkObjectExpression(c.Credentials.Expression, "credentials", k)
+		} else {
+			rule.checkString(c.Credentials.Username, k)
+			rule.checkString(c.Credentials.Password, k)
+		}
 	}
-	rule.checkEnv(c.Env, workflowKey+".env.<env_id>") // e.g. jobs.<job_id>.container.env.<env_id>
+	rule.checkEnv(c.Env, childWorkflowKey+".env.<env_id>") // e.g. jobs.<job_id>.container.env.<env_id>
 	rule.checkStrings(c.Ports, workflowKey)
 	rule.checkStrings(c.Volumes, workflowKey)
 	rule.checkString(c.Options, workflowKey)
@@ -572,6 +582,15 @@ func (rule *RuleExpression) checkWorkflowCall(c *WorkflowCall) {
 	}
 }
 
+func (rule *RuleExpression) checkSnapshot(s *Snapshot) {
+	if s == nil {
+		return
+	}
+	// XXX: Available contexts in `jobs.<job_name>.snapshot` is not defined in the document.
+	// https://github.com/github/docs/issues/41255
+	rule.checkIfCondition(s.If, "🐶")
+}
+
 func (rule *RuleExpression) checkWebhookEventFilter(f *WebhookEventFilter) {
 	if f == nil {
 		return
@@ -778,10 +797,16 @@ func (rule *RuleExpression) checkSemanticsOfExprNode(expr ExprNode, line, col in
 	if rule.jobsTy != nil {
 		c.UpdateJobs(rule.jobsTy)
 	}
-	if workflowKey != "" {
+	if workflowKey == "🐶" {
+		// XXX: This is a special case for `jobs.<job_name>.snapshot`. See `checkSnapshot` method
+		ctx := slices.Sorted(maps.Keys(AllContexts))
+		sp := slices.Sorted(maps.Keys(SpecialFunctionNames))
+		c.SetContextAvailability(ctx)
+		c.SetSpecialFunctionAvailability(sp)
+	} else if workflowKey != "" {
 		ctx, sp := WorkflowKeyAvailability(workflowKey)
 		if len(ctx) == 0 {
-			rule.Debug("No context avaiability was found for workflow key %q", workflowKey)
+			rule.Debug("No context availability was found for workflow key %q", workflowKey)
 		}
 		c.SetContextAvailability(ctx)
 		c.SetSpecialFunctionAvailability(sp)
