@@ -11,8 +11,10 @@ func TestRuleIfCond(t *testing.T) {
 		valid bool
 	}{
 		{"", true},
-		{"true", true},
+		{"true", false},
+		{"false", false},
 		{"true || false", true},
+		{"github.ref_name == 'foo'", true},
 		{"${{ false }}", true},
 		{"${{ false }}\n", false},
 		{"${{ false }} ", false},
@@ -47,6 +49,10 @@ func TestRuleIfCond(t *testing.T) {
 			var j Job
 			if len(tc.cond) > 0 {
 				j.If = &String{Value: tc.cond, Pos: &Pos{}}
+				j.Snapshot = &Snapshot{
+					ImageName: &String{Value: "test", Pos: &Pos{}},
+					If:        &String{Value: tc.cond, Pos: &Pos{}},
+				}
 			}
 
 			r := NewRuleIfCond()
@@ -58,7 +64,7 @@ func TestRuleIfCond(t *testing.T) {
 			if tc.valid && len(errs) > 0 {
 				t.Fatalf("wanted no error but have %q for condition %q", errs, tc.cond)
 			}
-			if !tc.valid && len(errs) != 1 {
+			if !tc.valid && len(errs) != 2 {
 				t.Fatalf("wanted one error but have %q for condition %q", errs, tc.cond)
 			}
 		})
