@@ -1,25 +1,27 @@
 package actionlint
 
-var allPermissionScopes = map[string]struct{}{
-	"actions":             {},
-	"attestations":        {},
-	"checks":              {},
-	"contents":            {},
-	"deployments":         {},
-	"discussions":         {},
-	"id-token":            {},
-	"issues":              {},
-	"models":              {},
-	"packages":            {},
-	"pages":               {},
-	"pull-requests":       {},
-	"repository-projects": {},
-	"security-events":     {},
-	"statuses":            {},
+import "slices"
+
+var allPermissionScopes = map[string][]string{
+	"actions":             {"read", "write", "none"},
+	"attestations":        {"read", "write", "none"},
+	"checks":              {"read", "write", "none"},
+	"contents":            {"read", "write", "none"},
+	"deployments":         {"read", "write", "none"},
+	"discussions":         {"read", "write", "none"},
+	"id-token":            {"write", "none"},
+	"issues":              {"read", "write", "none"},
+	"models":              {"read", "none"},
+	"packages":            {"read", "write", "none"},
+	"pages":               {"read", "write", "none"},
+	"pull-requests":       {"read", "write", "none"},
+	"repository-projects": {"read", "write", "none"},
+	"security-events":     {"read", "write", "none"},
+	"statuses":            {"read", "write", "none"},
 }
 
 // RulePermissions is a rule checker to check permission configurations in a workflow.
-// https://docs.github.com/en/actions/security-for-github-actions/security-guides/automatic-token-authentication#permissions-for-the-github_token
+// https://docs.github.com/en/actions/reference/workflows-and-actions/workflow-syntax#defining-access-for-the-github_token-scopes
 type RulePermissions struct {
 	RuleBase
 }
@@ -56,25 +58,25 @@ func (rule *RulePermissions) checkPermissions(p *Permissions) {
 		case "write-all", "read-all":
 			// OK
 		default:
-			rule.Errorf(p.All.Pos, "%q is invalid for permission for all the scopes. available values are \"read-all\" and \"write-all\"", p.All.Value)
+			rule.Errorf(p.All.Pos, "%q is invalid for permission for all the scopes. available values are \"read-all\", \"write-all\" or {}", p.All.Value)
 		}
 		return
 	}
 
 	for _, p := range p.Scopes {
 		n := p.Name.Value // Permission names are case-sensitive
-		if _, ok := allPermissionScopes[n]; !ok {
+		s, ok := allPermissionScopes[n]
+		if !ok {
 			ss := make([]string, 0, len(allPermissionScopes))
 			for s := range allPermissionScopes {
 				ss = append(ss, s)
 			}
 			rule.Errorf(p.Name.Pos, "unknown permission scope %q. all available permission scopes are %s", n, sortedQuotes(ss))
+			continue
 		}
-		switch p.Value.Value {
-		case "read", "write", "none":
-			// OK
-		default:
-			rule.Errorf(p.Value.Pos, "%q is invalid for permission of scope %q. available values are \"read\", \"write\" or \"none\"", p.Value.Value, n)
+
+		if !slices.Contains(s, p.Value.Value) {
+			rule.Errorf(p.Value.Pos, "%q is invalid as permission of scope %q. available values are %s", p.Value.Value, n, quotes(s))
 		}
 	}
 }
