@@ -152,7 +152,10 @@ func (g *gen) fetchRemote() (map[string]*actionlint.ActionMetadata, error) {
 					}
 					spec := req.action.spec(req.tag)
 					var meta actionlint.ActionMetadata
-					if err := yaml.Unmarshal(body, &meta); err != nil {
+					if err := yaml.Unmarshal(body, &meta); err != nil &&
+						// treosh/lighthouse-ci-action's metadata causes parse error due to typo. Ignore the error not to stop codegen
+						// https://github.com/treosh/lighthouse-ci-action/pull/153
+						!(strings.HasPrefix(spec, "treosh/lighthouse-ci-action@") && strings.Contains(err.Error(), "unexpected key")) {
 						ret <- &fetched{err: fmt.Errorf("could not parse metadata for %s: %w", url, err)}
 						break
 					}
@@ -277,7 +280,7 @@ var PopularActions = map[string]*ActionMetadata{
 			fmt.Fprintf(b, "Inputs: ActionMetadataInputs{\n")
 			for _, id := range ids {
 				i := meta.Inputs[id]
-				fmt.Fprintf(b, "%q: {%q, %v},\n", id, i.Name, i.Required)
+				fmt.Fprintf(b, "%q: {%q, %v, %v, %q},\n", id, i.Name, i.Required, i.Deprecated, i.DeprecationMessage)
 			}
 			fmt.Fprintf(b, "},\n")
 		}
